<?php

class WAT_Cache implements ArrayAccess
{
	private $dbh;
	private $queries;
	private $max_age;

	public function __construct ($dburi, $user, $pass, $table, $max_age, $dfield)
	{
		$this->dbh = new PDO ($dburi, $user, $pass);

		$this->queries = array (
			'fetch' => $this->dbh->prepare (
				'SELECT ' . $dfield . ' FROM ' . $table . ' WHERE cache_id = ? AND last_update > UNIX_TIMESTAMP(NOW()) - ' . $max_age
			),
			'store' => $this->dbh->prepare (
				'INSERT INTO ' . $table . ' (cache_id, ' . $dfield . ', last_update) VALUES (?, ?, UNIX_TIMESTAMP(NOW()))
					ON DUPLICATE KEY UPDATE ' . $dfield . '=VALUES(' . $dfield . '), last_update=VALUES(last_update)'
			),
			'delete' => $this->dbh->prepare ('DELETE FROM ' . $table . ' WHERE cache_id = ?'),
			'clean_cache' => $this->dbh->prepare (
				'DELETE FROM ' . $table . ' WHERE last_update <= UNIX_TIMESTAMP(NOW()) - ?'
			),
		);

		$this->max_age = $max_age;
	}

	public function clean ($max_age = -1) {
		if ($max_age < 0) {
			$max_age = $this->max_age;
		}
		$this->queries['clean_cache']->execute (array ($max_age));
		$r = $this->queries['clean_cache']->rowCount ();
		$this->queries['clean_cache']->closeCursor ();

		return $r;
	}

	public function offsetGet ($cache_id)
	{
		$this->queries['fetch']->execute (array ($cache_id));
		$t = $this->queries['fetch']->fetch (PDO::FETCH_NUM);
		$r = $t[0];
		$this->queries['fetch']->closeCursor ();

		return $r;
	}

	public function offsetSet ($cache_id, $data)
	{
		$r = $this->queries['store']->execute (array ($cache_id, $data));
		$this->queries['store']->closeCursor ();

		return $r;
	}

	public function offsetUnset ($cache_id)
	{
		$r = $this->queries['delete']->execute (array ($cache_id));
		$this->queries['delete']->closeCursor ();

		return $r;
	}

	public function offsetExists ($cache_id)
	{
		$r = $this->queries['fetch']->execute (array ($cache_id));
		$this->queries['fetch']->closeCursor ();

		return $r;
	}
};

?>
