<?php

class WAT_Char_ToolTip
{
	protected $char;
	protected $realm;
	protected $armory;

	protected $stats_conf = array (
		/* --base stats-- */
		'stamina' => FALSE,
		'strength' => FALSE,
		'intellect' => FALSE,
		'agility' => FALSE,
		'spirit' => FALSE,
		'armor' => FALSE,
		/*   -- spell damage--   */
		'shadow_power' => FALSE,
		'shadow_crit' => FALSE,
		'fire_power' => FALSE,
		'fire_crit' => FALSE,
		'frost_power' => FALSE,
		'frost_crit' => FALSE,
		'arcane_power' => FALSE,
		'arcane_crit' => FALSE,
		'nature_power' => FALSE,
		'nature_crit' => FALSE,
		'holy_power' => FALSE,
		'holy_crit' => FALSE,
		'healing' => FALSE,
		'mana_regen' => FALSE,
		'mana_regen_cast' => FALSE,
		'spell_hit' => FALSE,
		'penetration' => FALSE,
		/*   -- melee damage--   */
		'melee_main_dmg' => FALSE,
		'melee_main_speed' => FALSE,
		'melee_main_dps' => FALSE,
		'melee_off_dmg' => FALSE,
		'melee_off_speed' => FALSE,
		'melee_off_dps' => FALSE,
		'melee_power' => FALSE,
		'melee_hit' => FALSE,
		'melee_crit' => FALSE,
		'melee_expertise' => FALSE,
		/*   -- ranged damage--   */
		'ranged_power' => FALSE,
		'ranged_dmg' => FALSE,
		'ranged_speed' => FALSE,
		'ranged_dps' => FALSE,
		'ranged_crit' => FALSE,
		'ranged_hit' => FALSE,
                /*  --Haste--  */
                'haste_rating' => FALSE,
		/*  --defenses--  */
		'defense' => FALSE,
		'dodge' => FALSE,
		'parry' => FALSE,
		'block' => FALSE,
		'resilience' => FALSE,
		/*  --resistances--  */
		'arcane_resist' => FALSE,
		'fire_resist' => FALSE,
		'frost_resist' => FALSE,
		'shadow_resist' => FALSE,
		'nature_resist' => FALSE,
		'holy_resist' => FALSE,
	);

	private $talent_trees = array (
		'deathknight' => array ('Blood', 'Frost', 'Unholy'),
		'druid' => array ('Balance', 'Feral', 'Restoration'),
		'hunter' => array ('Beast Mastery', 'Marksmanship', 'Survival'),
		'mage' => array ('Arcane', 'Fire', 'Frost'),
		'paladin' => array ('Holy', 'Protection', 'Retribution'),
		'priest' => array ('Discipline', 'Holy', 'Shadow'),
		'rogue' => array ('Assassination', 'Combat', 'Subtlety'),
		'shaman' => array ('Elemental', 'Enhancement', 'Restoration'),
		'warlock' => array ('Affliction', 'Demonology', 'Destruction'),
		'warrior' => array ('Arms', 'Fury', 'Protection'),
	);

	private $id_to_name = array (
		'unknown',
		'warrior',
		'paladin',
		'hunter',
		'rogue',
		'priest',
		'deathknight',
		'shaman',
		'mage',
		'warlock',
		'(10)',
		'druid',
	);

	protected $armory_data;
	protected $url_base;
	protected $avatar_url_base;
	protected $icons_url_base;
	protected $char_data;
	protected $main_spec;

	public $error_message;

	protected $public_data = array ();

	public function __construct ($char, $realm, $armory, $url_base = NULL)
	{
		$this->char = $char;
		$this->realm = $realm;
		$this->armory = $armory;
		if ($url_base) {
			$this->url_base = $url_base;
		} else {
			$this->url_base = $this->armory->armory;
		}

		$this->error_message = null;

		$this->generate ();
	}

	private function enable_stats (array $stats)
	{
		foreach ($stats as $stat) {
			$this->stats_conf[$stat] = TRUE;
		}
	}

	private function grab_armory ()
	{
		// Get the char data from Armory
		$this->armory_data = $this->armory->characterFetch ($this->char, $this->realm);

		// Url for armory icons
		$this->avatar_url_base = $this->url_base . "images/portraits/";
		$this->icons_url_base  = $this->url_base . "images/icons/";

		// Reduce the array name size, make more easy to work with the array.
		$this->armory_data = $this->armory_data['characterinfo'];

		if (count ($this->armory_data) == 0) {
			$this->error_message = 'Error connecting to the armory.';
			return;
		}

		if ($this->armory_data['errcode'] == 'noCharacter') {
			$this->error_message = 'No such character.';
			return;
		}

		if (!$this->armory_data['charactertab']) {
			$this->error_message = 'Character data unavailable.';
			return;
		}

		$this->public_data['class'] = $this->id_to_name[$this->armory_data['character']['classid']];
	}

	private function process_armory_data ()
	{
		// To change the fields by class edit the stats_conf/ files
		// each class have a separated config.

		$char = $this->armory_data;

		$stamina = $char['charactertab']['basestats']['stamina']['effective'] . " (" . $char['charactertab']['basestats']['stamina']['base'] . " + " . ($char['charactertab']['basestats']['stamina']['effective'] - $char['charactertab']['basestats']['stamina']['base']) . ")";
		$intellect = $char['charactertab']['basestats']['intellect']['effective'] . " (" . $char['charactertab']['basestats']['intellect']['base'] . " + " . ($char['charactertab']['basestats']['intellect']['effective'] - $char['charactertab']['basestats']['intellect']['base']) . ")";
		$strength = $char['charactertab']['basestats']['strength']['effective'] . " (" . $char['charactertab']['basestats']['strength']['base'] . " + " . ($char['charactertab']['basestats']['strength']['effective'] - $char['charactertab']['basestats']['strength']['base']) . ")";
		$agility = $char['charactertab']['basestats']['agility']['effective'] . " (" . $char['charactertab']['basestats']['agility']['base'] . " + " . ($char['charactertab']['basestats']['agility']['effective'] - $char['charactertab']['basestats']['agility']['base']) . ")";
		$spirit = $char['charactertab']['basestats']['spirit']['effective'] . " (" . $char['charactertab']['basestats']['spirit']['base'] . " + " . ($char['charactertab']['basestats']['spirit']['effective'] - $char['charactertab']['basestats']['spirit']['base']) . ")";
		$armor = $char['charactertab']['basestats']['armor']['effective'] . " (" . $char['charactertab']['basestats']['armor']['base'] . " + " . ($char['charactertab']['basestats']['armor']['effective'] - $char['charactertab']['basestats']['armor']['base']) . ")";

		$extradata['health'] = Array  (
			'field' => 'Health',
			'value' => $char['charactertab']['characterbars']['health']['effective'],
			'class' => 'health',
		);
		$extradata['secondbar'] = Array (
			'field' => 'Power',
			'value' => $char['charactertab']['characterbars']['secondbar']['effective'],
			'class' => 'power_mana',
		);
		$extradata['stamina'] = Array (
			'field' => 'Stamina',
			'value' => $stamina,
			'class' => 'primary',
		);
		$extradata['intellect'] = Array (
			'field' => 'Intellect',
			'value' => $intellect,
			'class' => 'primary',
		);
		$extradata['strength'] = Array (
			'field' => 'Strength',
			'value' => $strength,
			'class' => 'primary',
		);
		$extradata['armor'] = Array (
			'field' => 'Armor',
			'value' => $armor,
			'class' => 'primary',
		);
		$extradata['agility'] = Array (
			'field' => 'Agility',
			'value' => $agility,
			'class' => 'primary',
		);
		$extradata['spirit'] = Array (
			'field' => 'Spirit',
			'value' => $spirit,
			'class' => 'primary',
		);

		foreach (array ('Shadow', 'Fire', 'Frost', 'Arcane', 'Nature', 'Holy') as $spell_school) {
			$extradata[strtolower ($spell_school) . '_power'] = array (
				'field' => $spell_school .' damage',
				'value' => $char['charactertab']['spell']['bonusdamage'][strtolower ($spell_school)]['value'],
				'class' => strtolower ($spell_school) . '_spell',
			);
			$extradata[strtolower ($spell_school) . '_crit'] = array (
				'field' => $spell_school . ' crit. chance',
				'value' => $char['charactertab']['spell']['critchance'][strtolower ($spell_school)]['percent']."%",
				'class' => strtolower ($spell_school) . '_spell',
			);
		}

		$extradata['spell_hit'] = Array (
			'field' => 'Spell hit chance',
			'value' => $char['charactertab']['spell']['hitrating']['increasedhitpercent'] . "%",
			'class' => 'generic',
		);
		
                $extradata['haste_rating'] = Array (
                        'field' => 'Haste rating',
                        'value' => $char['charactertab']['spell']['hasterating']['hasterating'] . ' / ' . $char['charactertab']['spell']['hasterating']['hastepercent'] . '%',
                        'class' => 'generic',
                );

		$extradata['penetration'] = Array (
			'field' => 'Spell penetration',
			'value' => $char['charactertab']['spell']['penetration']['value'],
			'class' => 'generic',
			);

		foreach (array ('main' ,'off') as $hand) {
			$extradata['melee_' . $hand . '_dmg'] = array (
				'field' => 'Melee ' . $hand . ' damage',
				'value' => $char['charactertab']['melee'][$hand . 'handdamage']['max'] . "-" .
				$char['charactertab']['melee'][$hand . 'handdamage']['min'],
				'class' => 'melee_' . $hand . '_hand',
			);
			$extradata['melee_' . $hand . '_speed'] = array (
				'field' => 'Melee ' . $hand . ' speed',
				'value' => $char['charactertab']['melee'][$hand . 'handdamage']['speed'],
				'class' => 'melee_' . $hand . '_hand',
			);
			$extradata['melee_' . $hand . '_dps'] = array (
				'field' => 'Melee ' . $hand . ' DPS',
				'value' => $char['charactertab']['melee'][$hand . 'handdamage']['dps'],
				'class' => 'melee_' . $hand . '_hand'
			);
		}

		$extradata['melee_power'] = Array (
			'field' => 'Melee attack power',
			'value' => $char['charactertab']['melee']['power']['effective'],
			'class' => 'generic',
		);
		$extradata['melee_hit'] = Array (
			'field' => 'Melee hit chance',
			'value' => $char['charactertab']['melee']['hitrating']['increasedhitpercent'] . '%',
			'class' => 'generic',
		);
		$extradata['melee_crit'] = Array(
			'field' => 'Melee crit. chance',
			'value' => $char['charactertab']['melee']['critchance']['percent'] . '%',
			'class' => 'generic',
		);
		$extradata['melee_expertise'] = Array(
			'field' => 'Melee expertise',
			'value' => $char['charactertab']['melee']['expertise']['value'],
			'class' => 'generic',
		);

		$extradata['defense'] = Array (
			'field' => 'Defense',
			'value' => $char['charactertab']['defenses']['defense']['value'] +
			$char['charactertab']['defenses']['defense']['plusdefense'],
			'class' => 'defensive',
		);

		foreach (array ('Block', 'Dodge', 'Parry') as $avoidance) {
			$extradata[strtolower ($avoidance)] = Array (
				'field' => $avoidance . ' chance',
				'value' => $char['charactertab']['defenses'][strtolower ($avoidance)]['percent'] . "%",
				'class' => 'defensive',
			);
		}

		$extradata['resilience'] = Array (
			'field' => 'Resilience',
			'value' => $char['charactertab']['defenses']['resilience']['value'],
			'class' => 'defensive',
		);

		$extradata['healing'] = Array (
			'field' => 'Healing',
			'value' => $char['charactertab']['spell']['bonushealing']['value'],
			'class' => 'healing',
			);
		$extradata['mana_regen'] = Array (
			'field' => 'Mana regen',
			'value' => $char['charactertab']['spell']['manaregen']['notcasting'],
			'class' => 'mana_regen',
			);
		$extradata['mana_regen_cast'] = Array (
			'field' => 'Mana regen (casting)',
			'value' => $char['charactertab']['spell']['manaregen']['casting'],
			'class' => 'mana_regen',
		);

		$extradata['ranged_dmg'] = Array (
			'field' => 'Ranged damage',
			'value' => $char['charactertab']['ranged']['damage']['max'] . "-" .$char['charactertab']['ranged']['damage']['min'],
			'class' => 'ranged',
		);
		$extradata['ranged_dps'] = Array (
			'field' => 'Ranged DPS',
			'value' => $char['charactertab']['ranged']['damage']['dps'],
			'class' => 'ranged',
		);
		$extradata['ranged_crit'] = Array (
			'field' => 'Ranged crit. chance',
			'value' => $char['charactertab']['ranged']['critchance']['percent'] ."%",
			'class' => 'ranged',
		);
		$extradata['ranged_speed'] = Array (
			'field' => 'Ranged speed',
			'value' => $char['charactertab']['ranged']['speed']['value'],
			'class' => 'ranged',
		);
		$extradata['ranged_hit'] = Array (
			'field' => 'Ranged hit chance',
			'value' => $char['charactertab']['ranged']['hitrating']['increasedhitpercent'] . "%",
			'class' => 'ranged',
		);
		$extradata['ranged_power'] = Array (
			'field' => 'Ranged attack power',
			'value' => $char['charactertab']['ranged']['power']['effective'],
			'class' => 'ranged',
		);

		$this->char_data = $extradata;
	}

	private function generate_talent_info ()
	{
		$info = $this->armory_data['charactertab']['talentspecs']['talentspec'];


                if ($info && !($info[0])) {
			$info[0] = $info;
		}


                for ($i = 0; $i <= 1; $i++) {
                        if ($info[$i]) {
				$tree = $info[$i];
	    		       	switch (max ($tree['treeone'], $tree['treetwo'], $tree['treethree'])) {
               				case $tree['treeone']: $main_tree = 1; break;
               				case $tree['treetwo']: $main_tree = 2; break;
               				case $tree['treethree']: $main_tree = 3; break;
               			}
				if ($info[$i]['active'] == "1") {
                                	$this->main_spec = $main_tree;
				}
				$this->public_data['talents'][$i] = array (
					'main_spec' => $this->talent_trees[$this->public_data['class']][$main_tree - 1],
					'icon_url' => $this->icons_url_base . 'class/' . $this->armory_data['character']['classid'] . '/talents/' . $main_tree . '.gif',
					'prim' => $tree['prim'],
					'active' => $tree['active'],
					'tree' => array (
						'main' => $main_tree,
						'1' => $tree['treeone'],
						'2' => $tree['treetwo'],
						'3'=> $tree['treethree'],
					)
				);
			}
		}
	}

	private function generate_profession_info ()
	{
		$info = $this->armory_data['charactertab']['professions']['skill'];

		for ($i = 0; $i <= 1; $i++) {
			if ($info[$i]) {
				$this->public_data['professions'][$i] = array (
					'icon_url' => $this->icons_url_base . 'professions/' . $info[$i]['key'] . '-sm.gif',
					'value' => $info[$i]['value'],
					'max' => $info[$i]['max'],
					'name' => $info[$i]['name'],
				);
			}
		}
	}

        private function generate_avatar_info ()
        {
                // Create the Avatar URL
                $char = $this->armory_data;

                $avatar = $this->avatar_url_base;
                if ($char['character']['level'] == 80) {
                        $avatar .= "wow-80/";
                } elseif ($char['character']['level'] == 70) {
                        $avatar .= "wow-70/";
                } elseif ($char['character']['level'] < 70 && $char['character']['level'] > 59) {
                        $avatar .= "wow/";
                } else {
                        $avatar .= "wow-default/";
                }
                $avatar .= $char['character']['genderid'] . '-' . $char['character']['raceid'] . '-' . $char['character']['classid'] . '.gif';

                $this->public_data['avatar']['url'] = $avatar;
        }


	private function generate_buff_info ()
	{
		$this->public_data['buffs'] = array ();

		if (is_array ($this->armory_data['charactertab']['buffs']['spell'][0]))	{
			foreach ($this->armory_data['charactertab']['buffs']['spell'] as $buf) {
				$this->public_data['buffs'][] = array (
					'name' => $buf['name'],
					'icon' => $this->icons_url_base . "/21x21/" . $buf['icon'] . '.png',
					'effect' => $buf['effect'],
				);
			}
		} elseif (isset ($this->armory_data['charactertab']['buffs']['spell']['icon'])) {
			$buf = $this->armory_data['charactertab']['buffs']['spell'];
			$this->public_data['buffs'][] = array (
				'name' => $buf['name'],
				'icon' => $this->icons_url_base . "/21x21/" . $buf['icon'] . '.png',
				'effect' => $buf['effect'],
			);
		}
	}

	private function finalise_char ()
	{
		include_once ('includes/class_conf/' . $this->public_data['class'] . '.php');

		$this->public_data['stats'] = array ();
		foreach ($this->stats_conf as $stat => $value) {
			if ($value)
				$this->public_data['stats'][$stat] = $this->char_data[$stat];
		}

		// define parameters for the class
		if ($this->public_data['class'] == 'warrior') $this->char_data['secondbar']['class'] = 'power_rage';
		if ($this->public_data['class'] == 'rogue') $this->char_data['secondbar']['class'] = 'power_energy';
	}

	private function generate ()
	{
		$this->grab_armory ();
		$this->process_armory_data ();

		$this->generate_talent_info ();
		$this->generate_profession_info ();
		$this->generate_avatar_info ();
		$this->generate_buff_info ();
		$this->finalise_char ();

		$this->public_data['health'] = $this->char_data['health']['value'];
		$this->public_data['secondbar'] = array ('value' => $this->char_data['secondbar']['value'],
							 'class' => $this->char_data['secondbar']['class']);
		$this->public_data['char'] = array (
			'name' => $this->armory_data['character']['name'],
			'title' => $this->armory_data['charactertab']['title']['value'],
			'guild' => ($this->armory_data['character']['guildname']) ? $this->armory_data['character']['guildname'] : " &nbsp; ",
			'level' => $this->armory_data['character']['level'],
			'race' => $this->armory_data['character']['race'],
			'class' => $this->armory_data['character']['class'],
		);
	}

	public function __get ($index)
	{
		$r = $this->public_data[$index];
		return $r;
	}
};
?>
