<?php

class WAT_Item_ToolTip
{
	private $bonding = array (
		0 => 'Unknown',
		1 => 'Binds when picked up',
		2 => 'Binds when equipped',
		3 => 'Soulbound',
		4 => 'Quest item',
		5 => 'Quest item',
	);

	private $slots = array (
		 0 => ' ',
		 1 => 'Head',
		 2 => 'Neck',
		 3 => 'Shoulders',
		 4 => 'Shirt',
		 5 => 'Chest',
		 6 => 'Waist',
		 7 => 'Legs',
		 8 => 'Feet',
		 9 => 'Wrist',
		10 => 'Hands',
		11 => 'Finger',
		12 => 'Trinket',
		13 => 'One-Hand',
		14 => 'Off-hand',
		15 => 'Ranged',
		16 => 'Back',
		17 => 'Two-hand',
		18 => 'Bag',
		19 => 'Tabard',
		20 => 'Chest',
		21 => 'Main-hand',
		22 => 'Off-hand',
		23 => 'Held in off-hand',
		24 => 'Projectile',
		25 => 'Thrown',
		26 => 'Ranged',
		27 => 'Quiver',
		28 => 'Relic',
	);

	private $reputation = array (
		0 => 'Hated',
		1 => 'Hostile',
		2 => 'Unfriendly',
		3 => 'Neutral',
		4 => 'Friendly',
		5 => 'Honored',
		6 => 'Revered',
		7 => 'Exalted',
	);

	private $spell_trigger = array (
		0 => 'on_use',
		1 => 'on_equip',
		2 => 'chance_on_hit',
	);

	private $on_equip_stats = array (
		'manaregen' => 'Increases your mana regen',
		'critspellrating' => 'Improves spell critical rating',
		'hitspellrating' => 'Improves spell hit rating',
		'hitrating' => 'Improves hit rating',
		'defenseskillrating' => 'Increases your defense rating',
		'parryrating' => 'Increases your parry rating',
		'dodgerating' => 'Increases your odge rating',
		'blockrating' => 'Increases your block rating',
		'critrating' => 'Improves critical strike rating',
		'resiliencerating' => 'Improves your resilience rating',
		'hitmeleerating' => 'Improves your melee hit rating',
		'hitrangedrating' => 'Improves your ranged hit rating',
		'critmeleerating' => 'Improves your melee critical strike rating',
		'critrangedrating' => 'Improves your ranged critical strike rating',
		'hastemeleerating' => 'Improves your melee haste rating',
		'hasterangedrating' => 'Improves your ranged haste rating',
		'hastespellrating' => 'Improves your spell haste rating',
		'hasterating' => 'Improves your haste rating',
		'spellpower' => 'Increases your spell power',
		'expertiserating' => 'Improves your expertise rating',
		'attackpower' => 'Increases your attack power',
		'armorpenetration' => 'Increases your armor penetration',
	);

	private $stats = array (
		'bonusstamina' => 'Stamina',
		'bonusintellect' => 'Intellect',
		'bonusspirit' => 'Spirit',
		'bonusagility' => 'Agility',
		'bonusstrength' => 'Strength',
		'blockvalue' => 'Block',
		'fireresist' => 'Fire resistance',
		'frostresist' => 'Frost resistance',
		'natureresist' => 'Nature resistance',
		'shadowresist' => 'Shadow resistance',
		'arcaneresist' => 'Arcane resistance',
	);

	private $qualities = array (
		0 => 'junk',
		1 => 'common',
		2 => 'uncommon',
		3 => 'rare',
		4 => 'epic',
		5 => 'legendary',
		6 => 'artifact',
	);

	private $damage_types = array (
		0 => 'Physical',
		1 => 'Holy',
		2 => 'Fire',
		3 => 'Nature',
		4 => 'Frost',
		5 => 'Shadow',
		6 => 'Arcane',
	);

	protected $public_data = array ();

	protected $item;
	protected $armory;
	protected $armory_data;
	protected $url_base;
	protected $icons_url_base;

	public $error_message;

	public function __construct ($item, $armory, $url_base = NULL)
	{
		$this->item = $item;
		$this->armory = $armory;
		if ($url_base) {
			$this->url_base = $url_base;
		} else {
			$this->url_base = $this->armory->armory;
		}

		$this->error_message = null;

		$this->generate ();
	}

	private function grab_armory ()
	{
		if (preg_match ('/^[0-9]+$/', $this->item) == 1) {
			$item = $this->armory->itemFetch ($this->item);
		} else {
			$item = $this->armory->itemNameFetch ($this->item);
		}

		// Url for armory icons
		$this->icons_url_base = $this->url_base . "wow-icons/_images/";
		$this->gem_icons_url_base = $this->url_base . "images/icons/";

		// Reduce the array name size, make more easy to work with the array.
		$item = $item['itemtooltips']['itemtooltip'];

		if (count ($item) == 0) {
			$this->error_message = 'Error connecting to the armory, or item not found.';
			return;
		}

		$this->armory_data = $item;
	}

	private function generate_required ()
	{
		$item = $this->armory_data;

		if (isset ($item['requiredability'])) {
			$this->public_data['required']['ability'] = $item['requiredability'];
		}

		if (isset ($item['requiredlevel'])) {
			$this->public_data['required']['level'] = $item['requiredlevel'];
		}

		if (isset ($item['requiredfaction'])) {
			$this->public_data['required']['faction'] = array (
				'class' => 'required_faction',
				'name' => $item['requiredfaction']['name'],
				'value' => $this->reputation[$item['requiredfaction']['rep']],
			);
		}

		if (isset ($item['requiredskill'])) {
			$this->public_data['required']['skill'] = array (
				'class' => 'required_skill',
				'name' => $item['requiredskill']['name'],
				'value' => $item ['requiredskill']['rank'],
			);
		}

		if (isset ($item['allowableclasses'])) {
			if (is_array($item['allowableclasses']['class'])) {
				$this->public_data['required']['classes'] = $item['allowableclasses']['class'];
			} else {
				$this->public_data['required']['classes'] = array ($item['allowableclasses']['class']);
			}
		}

		if ($item['maxcount'] == 1) {
			$this->public_data['required']['unique'] = TRUE;
		} else {
			$this->public_data['required']['unique'] = FALSE;
		}
	}

	private function build_spell_data ()
	{
		$item = $this->armory_data;
		$spell_data = array ();
		$spell_triggers = array ();

		if (isset ($item['spelldata'])) {
			if (is_array ($item['spelldata']['spell']['desc'])) {
				$spell_data = $item['spelldata']['spell']['desc'];
			} else {
				$spell_data = array ($item['spelldata']['spell']['desc']);
			}

			if (is_array ($item['spelldata']['spell']['trigger'])) {
				$spell_triggers = $item['spelldata']['spell']['trigger'];
			} else {
				$spell_triggers = array ($item['spelldata']['spell']['trigger']);
			}
		}

		for ($i = 0; $i < count ($spell_data); $i++) {
			$this->public_data['bonus_spell'][] = array (
				'trigger' => (isset ($spell_triggers[$i])) ?
					$this->spell_trigger [$spell_triggers[$i]] : $this->spell_trigger[0],
				'desc' => $spell_data[$i],
			);
		}
	}

	private function build_armor_dura_data ()
	{
		$item = $this->armory_data;

		if (isset ($item['armor'])) {
			$this->public_data['stats']['armor'] = $item['armor'][1];
		}
		if (isset ($item['durability'])) {
			$this->public_data['durability'] = $item['durability'];
		}
	}

	private function build_stats_data ()
	{
		$item = $this->armory_data;

		foreach ($this->stats as $key => $name) {
			if (isset ($item[$key])) {
				$this->public_data['stats'][$key] = array (
					'name' => $name,
					'value' => $item[$key]
				);
			}
		}
	}

	private function build_on_equip_data ()
	{
		$item = $this->armory_data;

		foreach ($this->on_equip_stats as $key => $bonus) {
			if (isset ($item['bonus' . $key])) {
				$this->public_data['bonus'][$key] = array (
					'bonus' => $bonus,
					'value' => $item['bonus' . $key],
				);
			}
		}
	}

	private function build_gem_data ()
	{
		$item = $this->armory_data;

		if (isset ($item['socketdata'])) {
		        // If exist more then one gem slot
		        if (is_array($item['socketdata']['socket'][0])) {
		                foreach($item['socketdata']['socket'] as $x) {
					$this->public_data['sockets'][] = array (
						'icon_url' => $this->gem_icons_url_base . 'Socket_' . $x['color'] . '.png',
						'color' => $x['color'],
					);
		        	}
			} else {
				$x = $item['socketdata']['socket'];
				$this->public_data['sockets'][] = array (
					'icon_url' => $this->gem_icons_url_base . 'Socket_' . $x['color'] . '.png',
					'color' => $x['color'],
				);
			}

			if ($item['socketdata']['socketmatchenchant']) {
				$this->public_data['socket_bonus'] = $item['socketdata']['socketmatchenchant'];
			}
		}

		if (isset($item['gemproperties'])) {
			$this->public_data['gemproperties'] = $item['gemproperties'];
		}
	}

	private function build_set_data ()
	{
		$item = $this->armory_data;

		if (!isset ($item['setdata']))
			return;

		$this->public_data['set_data'] = $item['setdata'];

		if (!is_array ($item['setdata']['setbonus'][0])) {
			$this->public_data['set_data']['setbonus'] = array ($item['setdata']['setbonus']);
		}
	}

	private function build_basic_item_data ()
	{
		$item = $this->armory_data;

		if (isset ($item['desc'])) {
			$this->public_data['desc'] = $item['desc'];
		}

		if ((isset ($item['equipdata']['inventorytype']) ||
		     isset ($item['equipdata']['subclassname'])) &&
		    !is_array ($item['equipdata']['inventorytype'])) {
			$this->public_data['equip'] = array (
				'slot' => $this->slots[$item['equipdata']['inventorytype']],
				'class' => $item['equipdata']['subclassname'],
			);
		}

		if (isset ($item['damagedata']['damage'])) {
			//$this->public_data['damage'] = $item['damagedata']['damage'];

			$dmg = $item['damagedata']['damage'];

			if (is_array ($dmg['type']) && count ($dmg['type']) == 0) {
				// Physical damage
				$this->public_data['damage']['damage'] = array (
					0 => array (
						type => $this->damage_types[0],
						min => $dmg['min'],
						max => $dmg['max'],
					),
				);
			} else if (!is_array ($dmg['type']) && is_array ($dmg['min'])) {
				// 1 Physical + 1 magic
				$this->public_data['damage']['damage'] = array (
					0 => array (
						type => $this->damage_types[0],
						min => $dmg['min'][0],
						max => $dmg['max'][0],
					),
					1 => array (
						type => $this->damage_types[$dmg['type']],
						min => $dmg['min'][1],
						max => $dmg['max'][1],
					),
				);
			} else {
				// Magical damage
				$this->public_data['damage']['damage'] = array (
					0 => array (
						type => $this->damage_types[$dmg['type']],
						min => $dmg['min'],
						max => $dmg['max'],
					),
				);
			}
			
			$this->public_data['damage']['speed'] = $item['damagedata']['speed'];
			$this->public_data['damage']['dps'] = round ($item['damagedata']['dps'], 2);
		}

		if (isset($item['bonding']) && !is_array ($item['bonding'])) {
			$this->public_data['bonding'] = $this->bonding[$item['bonding']];
		}

		if (is_array ($item['overallqualityid'])) {
			$item['overallqualityid'] = 0;
		}
		
		$this->public_data['item'] = array (
			'quality' => $this->qualities[$item['overallqualityid']],
			'name' => $item['name'],
        		'icon' => $this->icons_url_base . '43x43/' . $item['icon'] . '.png',
        	);
	}

	private function generate ()
	{
		$this->grab_armory ();
		$this->generate_required ();
		$this->build_spell_data ();
		$this->build_on_equip_data ();
		$this->build_stats_data ();
		$this->build_armor_dura_data ();
		$this->build_gem_data ();
		$this->build_set_data ();
		$this->build_basic_item_data ();

	}

	public function __get ($index)
	{
		$r = $this->public_data[$index];
		return $r;
	}
}
?>
